@@ -1,12 +1,13 @@
-#' Calculate various diversity and evenness measures
+#' Calculate various diversity measures
 #'
-#' Function to calculate different common measures of diversity and evenness.
-#' This includes Hill diversity, Functional Hill diversity,
-#' Shannon's diversity, Simpson diversity, Rao's Q, Pielou's evenness and
-#' Hill evenness.
+#' Function to calculate different common measures of diversity, and
+#' components (richness, evenness, disparity) thereof. Types of measures that
+#' can be calculated includes Hill diversity, Functional Hill diversity,
+#' Mean Pairwise Dissimilarity (MPD), Shannon's diversity, Simpson diversity,
+#' Rao's Q, Pielou's evenness and Hill evenness.
 #'
-#' The function calculates diversity and/or evenness for each sample
-#' in \code{sampleData}. It can calculate the following indices:
+#' The function calculates diversity and/or components thereof for each
+#' sample in \code{sampleData}. It can calculate the following indices:
 #' \itemize{
 #' \item \code{Shannon}. Shannon's Diversity.
 #' \item \code{Simpson}. Simpson Diversity, often referred to as
@@ -16,26 +17,30 @@
 #' species (here compounds). The parameter *q* determines the sensitivity
 #' of the measure to the relative frequencies of
 #' compounds (see above for details). For \code{q = 0}, this equals the
-#' number of compounds in a sample. For \code{q = 1}, this equals the
-#' exponential of Shannon's Diversity. For \code{q = 2}, this equals the
-#' Simpson Diversity.
-#' \item \code{FuncHillDiv}. Functional Hill Diversity. Equation 4b/6b in
-#' Chiu & Chao 2014, which is the measure called "total functional diversity".
-#' Requires a compound dissimilarity matrix. Functional Hill Diversity
-#' quantifies the effective total dissimilarity between compounds
-#' in the sample. The parameter *q* determines the sensitivity of
+#' number of compounds in a sample, i.e. the richness. For \code{q = 1},
+#' this equals the exponential of Shannon's Diversity. For \code{q = 2},
+#' this equals the Simpson Diversity.
+#' \item \code{FuncHillDiv}. Functional Hill Diversity. There are
+#' different versions of this. Here, the most common version is calculated.
+#' This is the FD(Q), called "total functional diversity", calculated in
+#' equation 4b/6b in Chiu & Chao 2014. This measure quantifies the
+#' effective total dissimilarity between compounds in the sample.
+#' Calculation of Functional Hill Diversity requires a compound
+#' dissimilarity matrix. The parameter *q* determines the sensitivity of
 #' the measure to the relative frequencies of compounds
 #' (see above for details). For \code{q = 1}, this is a measure
 #' sensitive to compound richness, evenness and dissimilarity, and is therefore
 #' the most comprehensive measure of diversity. For \code{q = 0}, this is
 #' equal to Functional Attribute Diversity (FAD) which is the sum of all
-#' dissimilarities in the dissimilarity matrix. FAD divided by
-#' *n(n-1)*, where *n* is the number of compounds and hence the number of
-#' rows/columns in the dissimilarity matrix, is equal to the
-#' Mean Pairwise Dissimilarity (MPD). This value is the mean of the
-#' pairwise dissimilarities in the compound dissimilarity matrix (excluding
-#' the 0 values in the diagonal), and is therefore in contrast to FAD not
-#' dependent on the number of compounds.
+#' dissimilarities in the dissimilarity matrix.
+#' \item \code{MPD}. Mean Pairwise Dissimilarity. As the name suggests, this
+#' is equal to the mean of the pairwise dissimilarities in the compound
+#' dissimilarity  matrix (excluding the 0 values in the diagonal). Therefore,
+#' in contrast to FAD (see above) this measure is not dependent on the
+#' number of compounds, and hence represents the disparity component
+#' of diversity. Practically, MPD is calculated as Functional Hill Diversity
+#' at \code{q = 0}, divided by *n(n-1)*, where *n* is the number of compounds,
+#' i.e. Hill Diversity at \code{q = 0}.
 #' \item \code{RaoQ}. Rao's quadratic entropy index Q.
 #' The perhaps most common measure of functional diversity.
 #' Requires a compound dissimilarity matrix. Rao's Q represents the
@@ -62,9 +67,9 @@
 #' @param compDisMat Compound dissimilarity matrix, as calculated by
 #' \code{\link{compDis}}. Has to be supplied for calculations of
 #' Functional Hill Diversity and Rao's Q.
-#' @param type Type(s) of diversity or evenness to calculate. Any of
-#' \code{"Shannon", "Simpson", "HillDiv", "FuncHillDiv", "RaoQ",
-#' "PielouEven", "HillEven"}.
+#' @param type Type(s) of diversity or component(s) thereof to calculate.
+#' Any of \code{"Shannon", "Simpson", "HillDiv", "FuncHillDiv", "MPD",
+#' "RaoQ", "PielouEven", "HillEven"}.
 #' @param q Diversity order to use for Hill diversity, Functional Hill
 #' Diversity and Hill Evenness. *q* should be equal to or larger than zero.
 #' This parameter determines the sensitivity of the (Functional) Hill Diversity
@@ -74,7 +79,7 @@
 #' weighed according to their proportion in the sample. For \code{q = 2},
 #' more weight is put on compounds with high proportions.
 #'
-#' @return Data frame with calculated diversity/evenness values for each sample.
+#' @return Data frame with calculated diversity values for each sample.
 #'
 #' @export
 #'
@@ -91,6 +96,17 @@
 #' Hill MO. 1973. Diversity and Evenness: A Unifying Notation
 #' and Its Consequences. Ecology 54: 427-432.
 #'
+#' Petren H, Koellner TG, Junker RR. 2023. Quantifying chemodiversity
+#' considering biochemical and structural properties of compounds with the
+#' R package *chemodiv*. New Phytologist 237: 2478-2492.
+#'
+#' Petren H, Anaia RA, Aragam KS, Braeutigam A, Eckert S, Heinen R,
+#' Jakobs R, Ojeda L, Popp M, Sasidharan R, Schnitzler J-P, Steppuhn A,
+#' Thon F, Tschikin S, Unsicker SB, van Dam NM, Weisser WW, Wittmann MJ,
+#' Yepes S, Ziaja D, Meuller C, Junker RR. 2023. Understanding the
+#' phytochemical diversity of plants: Quantification, variation and
+#' ecological function. bioRxiv doi: 10.1101/2023.03.23.533415.
+#'
 #' Tuomisto H. 2012. An updated consumer's guide to evenness
 #' and related indices. Oikos 121: 1203-1218
 #'
@@ -111,14 +127,16 @@ calcDiv <- function(sampleData,
                     type = "HillDiv",
                     q = 1) {
 
-  if (!(any(c("HillDiv", "FuncHillDiv", "Shannon", "Simpson",
+  if (!(any(c("HillDiv", "FuncHillDiv", "MPD", "Shannon", "Simpson",
               "PielouEven", "HillEven", "RaoQ") %in% type))) {
-    stop("Provide at least one type of diversity/evenness to calculate:
-         HillDiv, FuncHillDiv, Shannon, Simpson, PielouEven, HillEven or RaoQ.")
+    stop("Provide at least one type of measure to calculate:
+         HillDiv, FuncHillDiv, MPD, Shannon, Simpson, PielouEven, HillEven or RaoQ.")
   }
-  if(is.null(compDisMat) && ("FuncHillDiv" %in% type || "RaoQ" %in% type)) {
+  if(is.null(compDisMat) && ("FuncHillDiv" %in% type ||
+                             "MPD" %in% type ||
+                             "RaoQ" %in% type)) {
     stop("A compound dissimilarity matrix must be supplied
-         when calculating Functional Hill diversity or Rao's Q.")
+         when calculating Functional Hill diversity, MPD or Rao's Q.")
   }
   if(q < 0) {
     stop("q must be >= 0")
@@ -132,7 +150,7 @@ calcDiv <- function(sampleData,
   }
   if (any(c("HillDiv", "Shannon", "Simpson",
             "PielouEven", "HillEven") %in% type) &&
-      !any(c("FuncHillDiv", "RaoQ") %in% type) &&
+      !any(c("FuncHillDiv", "MPD", "RaoQ") %in% type) &&
       !is.null(compDisMat)) {
     message("Note that the calculated diveristy measures do not use the
             compound dissimilarity matrix.")
@@ -156,6 +174,20 @@ calcDiv <- function(sampleData,
                                             q = q)
     }
   }
+  if ("MPD" %in% type) {
+    tempFHD <- rep(NA, nrow(sampleData))
+
+    for (i in 1:nrow(sampleData)) {
+      tempFHD[i] <- funcHillDiv(data = sampleData[i,],
+                                Dij = compDisMat,
+                                q = 0)
+    }
+
+    tempHD <- hillR::hill_taxa(comm = sampleData,
+                               q = 0)
+
+    divData$MPD <- tempFHD / (tempHD * (tempHD - 1))
+  }
   if ("Shannon" %in% type) {
     divData$Shannon <- vegan::diversity(sampleData,
                                         index = "shannon",
